C--------------------------------------------------------------------
C The user of the OPEN-ADAS system and data (hereinafter "the User")
C accepts the following terms and conditions: 
C 
C   1. The University of Strathclyde uses all reasonable endeavours
C      to ensure the accuracy of the data provided and any
C      information given, but the University makes no warranty,
C      expressed or implied as to its accuracy, integrity, fitness
C      for purpose and shall not be responsible for the use to
C      which the data is put by the User and/or any consequences
C      arising out of any inaccuracies or omissions.
C 
C   2. Any downloaded OPEN-ADAS file is made available here for the
C      personal use of the User only.  It must not be used for any
C      commercial application, incorporated into any web site or in
C      any form re-distributed without express prior written
C      permission from the ADAS Project. In particular, but not by
C      way of limitation, it must not be:
C 
C        * inserted into a managed database structure
C        * redistributed along with a modelling or analysis code
C        * made available on a public website.
C 
C Permission for OPEN-ADAS data incorporation in non-commercial
C code/system development for scientific advance will not be
C unreasonably withheld, but will be subject to written agreement
C on a case by case basis.
C 
C The User will indemnify the University of Strathclyde and keep
C it fully and effectively indemnified against each and every
C claim made against the University as a result of the User's
C use of the data.  
C--------------------------------------------------------------------
       PROGRAM TEST
       
       IMPLICIT NONE
C------------------------------------------------------------------------
C  Purpose:
C          Test the XXDATA_01 routine and also provide a brief
C          example of its use.
C
C  Explanaton:
C          This program is distributed as part of the ADAS
C          subroutine library either via direct CVS to ADAS Project
C          members or via download by OPEN-ADAS users. It reads a
C          file called "test.dat" (distributed along with this
C          program) and prints out some of the contents. The
C          program is intended to serve as an example of using the
C          XXDATA_01 subroutine as well as testing it. A short
C          shell script called "test.sh" is used to compile and
C          test the code.
C
C  Variables:
C          All variables are documented by XXDATA_01. See either the
C          Fortran source or the PDF documentation for the routine.
C          Variables not passed into XXDATA_01 are trivial in nature.
C
C  Routines:
C          Routine    Source    Brief description
C          -------------------------------------------------------------
C          XXDATA_01  ADAS      Read complete data from an ADF01 file
C
C  Author : Allan Whiteford
C
C  Date   : 11/01/08
C       
C------------------------------------------------------------------------       

C-----------------------------------------------------------------------
C     Parameters required by XXDATA_01
C     --------------------------------        
      INTEGER   MXNENG , MXNSHL  
      PARAMETER (MXNENG = 30 , MXNSHL = 60)
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Input variables (i.e. non-parameter) required by XXDATA_01
C     ----------------------------------------------------------
      INTEGER   IUNIT
C-----------------------------------------------------------------------		

C-----------------------------------------------------------------------
C     Output variables required by XXDATA_01
C     --------------------------------------
      INTEGER   IZR     , IZD     , INDD     ,
     &          NENRGY  , NMIN    , NMAX
      LOGICAL   LPARMS  , LSETL
      CHARACTER SYMBR*2    , SYMBD*2
      INTEGER   LFORMA(MXNENG)
      REAL*8    ENRGYA(MXNENG)  , ALPHAA(MXNENG)  , XLCUTA(MXNENG)  ,
     &          PL2A(MXNENG)    , PL3A(MXNENG)    , SIGTA(MXNENG)
      REAL*8    SIGNA(MXNENG,MXNSHL)                           ,
     &          SIGLA(MXNENG,(MXNSHL*(MXNSHL+1))/2)
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Variables only used by this test program
C     ----------------------------------------
      INTEGER   IE      , IN
      CHARACTER*80 DSNIN     
C----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Open input file for reading
C     ---------------------------      
      IUNIT = 10
      DSNIN='test.dat'
      OPEN(UNIT = IUNIT , FILE=DSNIN , STATUS = 'OLD')
C-----------------------------------------------------------------------
       
C-----------------------------------------------------------------------
C     Pass unit number and dimension parameters to XXDATA_01 and
C     get back contents of file.
C     --------------------------      

      CALL XXDATA_01( IUNIT  , MXNENG , MXNSHL ,
     &                SYMBR  , SYMBD  , IZR    , IZD    ,      
     &                INDD   , NENRGY , NMIN   , NMAX   ,      
     &                LPARMS , LSETL  , ENRGYA ,               
     &                ALPHAA , LFORMA , XLCUTA , PL2A   ,      
     &                PL3A   , SIGTA  , SIGNA  , SIGLA         
     &              )
C-----------------------------------------------------------------------
   
C-----------------------------------------------------------------------
C     Print out some of the contents to screen
C     ----------------------------------------      

      PRINT *,'XXDATA_01 Test Program'      
      PRINT *,'----------------------'
      PRINT *,' '
      PRINT *,'The following is some information about the'
      PRINT *,'input file: '//DSNIN
      PRINT *,' '       
      PRINT '(A8,A)','DSNIN = ', DSNIN                                         
      PRINT '(A8,I4,A9,I4)',
     &      'MXNENG = ',MXNENG,' MXNSHL = ',  MXNSHL                                           
      PRINT '(A8,A4,A9,A4)',
     &      'SYMBR  = ',SYMBR, ' SYMBD  = ',  SYMBD                                            
      PRINT '(A8,I4,A9,I4)',
     &      'IZR    = ',IZR,   ' IZD    = ',  IZD                                              
      PRINT '(A8,I4,A9,I4)',
     &      'INDD   = ',INDD,  ' NENRGY = ',  NENRGY
      PRINT '(A8,I4,A9,I4)',
     &      'NMIN   = ',NMIN,  ' NMAX   = ',  NMAX                                             
      PRINT '(A8,L4,A9,L4)',
     &      'LPARMS = ',LPARMS,' LSETL  = ',  LSETL
     
      PRINT *
      PRINT '(A29,A11)','  IE  ENRGY(IE)   SIGTA(IE)  ',
     &      ' ALPHAA(IE)'          
      DO IE=1,NENRGY,4
        PRINT '(I4,1P3D12.4)',
     &        IE, ENRGYA(IE),SIGTA(IE),ALPHAA(IE)
      ENDDO
      
      PRINT *
      PRINT '(A33)','  IE  IN  ENRGY(IE)  SIGNA(IE,IN)'          
      DO IE=1,NENRGY,4
        DO IN=NMIN,NMAX,4
          PRINT '(2I4,1P2D12.4)',
     &          IE, IN,ENRGYA(IE),SIGNA(IE,IN)
        ENDDO
      ENDDO
C-----------------------------------------------------------------------

      END					  	
